///////////////////////////////////////////////////////////
///  ChameleonPendant                                   ///
///  (c) Koji Tsukada 2008                              ///
///////////////////////////////////////////////////////////

#include ".\ChameleonPendant.h"
#include ".\ColorSensor.c"
#include ".\ColorLed.c"

//a0: <analog> volume (not used in this version)
//a1: <analog> ir in
//a2: <out> color S2
//a3: <out> color S3
//a4: <out> ir led
//b0: <in> button
//b1: <in> color OUT
//b2: <out> LED  CS
//b3: <out> LED CLK
//b4: <out> white LED
//b5: <out> LED Data
//b6: <debug> ICD PGC
//b7: <debug> ICD PGD

//////////////////////////////////////////////////////////////////
void init_register()
{
   setup_adc_ports(sAN0|sAN1|VSS_VDD);
   setup_adc(ADC_CLOCK_INTERNAL|ADC_TAD_MUL_0);
   setup_wdt(WDT_OFF);
   setup_timer_0(RTCC_INTERNAL);
   setup_timer_1(T1_DISABLED);
   setup_timer_2(T2_DIV_BY_16,0,1);
   setup_timer_3(T3_DISABLED|T3_DIV_BY_1);

   setup_low_volt_detect(FALSE);
   setup_oscillator(False);

	set_tris_a(0x03);  //(XXX0 0011b) input = 1 output = 0
	set_tris_b(0xC3);  //(1100 0011b)
}

void init_variable()
{
   int p;
   for(p=0;p<UNIT_MAX;p++)
   {
      reds[p] = 0;
      greens[p] = 0;
      blues[p] = 0;
      red_leds[p] = 0;
      green_leds[p] = 0;
      blue_leds[p] = 0;
      red_leds_pre[p] = 0;
      green_leds_pre[p] = 0;
      blue_leds_pre[p] = 0;
      red_references[p] = 0;
      green_references[p] = 0;
      blue_references[p] = 0;
      init_flags[p] = 0;
      release_flags[p] = 1;

      turn_light(p, true);
      turn_ir(p,true);
   }
}

////////////////////////////////////////////////////////////////////////////
void main()
{
   int p = 0;
   int i = 0;
   init_register();
   init_variable();

   turn_light(0, TRUE);
   delay_ms(50);
   turn_light(0, FALSE);

   while(true)
   {
      for(p=0;p<UNIT_MAX;p++)
      {
         if(!input(PUSH_SWITCH))
         {
            if(release_flags[p])
            {
               release_flags[p] = 0;
               turn_ir(p, false);
               if(!init_flags[p])
               {
                  turn_light(p,true);
                  set_led(p,0,0,0);
                  set_white_reference(p);
                  turn_light(p,false);
                  init_flags[p] = 1;
                  turn_ir(p, true);
               }
               else
               {
                  turn_light(p,true);
                  set_led(p,0,0,0);
                  get_colors(p);
                  get_colors_led(p);
                  turn_light(p,false);
                  turn_ir(p,true);
                  set_led(p,red_leds[p],green_leds[p],blue_leds[p]);
               }
            }
         }
         else{
            release_flags[p] = 1;
         }
      }
   }
}
