/////////////////////////////////////////////////////////////////////////////
/// Controlling Color Sensor(TAOS230)                                ////////
////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//turn on/off the white LED.
void turn_light(int port, short flag){
   switch(port){
      case 0:
       if(flag)
         output_high(LED_WHITE_1);
       else
         output_low(LED_WHITE_1);
       break;
   }
   return;
}

//turn on/off the IR LED (proximity sensor).
void turn_ir(int port, short flag){
   switch(port){
      case 0:
       if(flag)
         output_high(IR_LED_1);
       else
         output_low(IR_LED_1);
       break;
   }
   return;
}

//check the proximity sensor
short check_ir_in(int port){
   short flag = 0;
   switch(port){
      case 0:
         flag =  input(IR_IN_1);
         break;
   }
   return flag;
}

//////////////////////////////////////////////////////////////////////////////
//count pulse of the color sensor
long count_color(int port, long period){
//period = msec, interval = 4usec, one period = 250 times
   long result = 0;
   long i = 0;
   long j = 0;

   short now = 0;
   short pre = 0;

   for(i = 0;i< period;i++){
      for(j=0;j<20;j++){
         switch(port){
            case 0:
               now = input(TAOS_OUT_1);
               break;
         }
         if(now != pre){
            if(now == 0){
               result += 1;
            }
            pre = now;
         }
         delay_us(4);
      }
   }
   return result;
}

void select_red(int port){
   switch(port){
      case 0:
         output_low(TAOS_S2_1);
         output_low(TAOS_S3_1);
         break;
   }
}

void select_green(int port){
   switch(port){
      case 0:
         output_high(TAOS_S2_1);
         output_high(TAOS_S3_1);
         break;
   }
}

void select_blue(int port){
   switch(port){
      case 0:
         output_low(TAOS_S2_1);
         output_high(TAOS_S3_1);
         break;
   }
}

void select_clear(int port){
   switch(port){
      case 0:
         output_high(TAOS_S2_1);
         output_low(TAOS_S3_1);
         break;
   }
}


void find_max(int port){
   int i = 0;
   int upper_limit = 255;
   int lower_limit = 0;

   for(i = 0;i<BISECT_TRIES;i++)
   {
      x = (int)((upper_limit - lower_limit) / 2.0) + lower_limit;

      count_read = count_color(port, x);

      if(count_read < DARK_OFFSET)
      {
           lower_limit = x;
      }
      else
      {
         count_read -= DARK_OFFSET;
         if(count_read > MAX_COLOR_VALUE)
         {
            upper_limit = x;
         }
         else if(count_read < MAX_COLOR_VALUE)
         {
            lower_limit = x;
         }
         else
         {
            break;
         }
      }
   }
   return;
}

void adjust_fine_frequency(int port)
{
   while(true){
      x -= 1;
      count_read = count_color(port,x);
      if(count_read < MAX_COLOR_VALUE)
         break;
   }
   while(true){
      x += 1;
      count_read = count_color(port,x);
      if(count_read >= MAX_COLOR_VALUE)
         break;
   }
   return;
}

//adjusting white balance on startup
void set_white_reference(int port){
   select_red(port);
   find_max(port);
   adjust_fine_frequency(port);
   red_references[port] = x;

   select_green(port);
   find_max(port);
   adjust_fine_frequency(port);
   green_references[port] = x;

   select_blue(port);
   find_max(port);
   adjust_fine_frequency(port);
   blue_references[port] = x;

   return;
}

//getting colors
void get_colors(int port){
   select_red(port);
   x= count_color(port, red_references[port]);
   if(x > MAX_COLOR_VALUE){
      reds[port] = MAX_COLOR_VALUE;
   }
   else{
      reds[port] = x;
   }

   select_green(port);
   x = count_color(port,green_references[port]);
   if(x > MAX_COLOR_VALUE){
      greens[port] = MAX_COLOR_VALUE;
   }
   else{
      greens[port] = x;
   }

   select_blue(port);
   x = count_color(port,blue_references[port]);
   if(x > MAX_COLOR_VALUE)
   {
      blues[port] = MAX_COLOR_VALUE;
   }
   else
   {
      blues[port] = x;
   }
   return;
}
